@@ -1,14 +1,39 @@
 # coding: utf-8
+import os
+
 from beaker.cache import clsmap, Cache
-from beaker.middleware import CacheMiddleware
+from beaker.middleware import CacheMiddleware, SessionMiddleware
 from nose import SkipTest
 from webtest import TestApp
 
-if 'ext:memcached' not in clsmap:
+if isinstance(clsmap.get('ext:memcached'), Exception):
     raise SkipTest("'memcache' or 'cmemcache' is not installed, can't test "
                    "memcached backend")
 
 mc_url = '127.0.0.1:11211'
+loc = os.path.sep.join([os.path.dirname(os.path.abspath(__file__)), 'sessions'])
+
+def simple_session_app(environ, start_response):
+    session = environ['beaker.session']
+    sess_id = environ.get('SESSION_ID')
+    if environ['PATH_INFO'].startswith('/invalid'):
+        # Attempt to access the session
+        id = session.id
+        session['value'] = 2
+    else:
+        if sess_id:
+            session = session.get_by_id(sess_id)
+        if not session:
+            start_response('200 OK', [('Content-type', 'text/plain')])
+            return ["No session id of %s found." % sess_id]
+        if not session.has_key('value'):
+            session['value'] = 0
+        session['value'] += 1
+        if not environ['PATH_INFO'].startswith('/nosave'):
+            session.save()
+    start_response('200 OK', [('Content-type', 'text/plain')])
+    return ['The current value is: %d, session id is %s' % (session['value'],
+                                                            session.id)]
 
 def simple_app(environ, start_response):
     extra_args = {}
@@ -29,6 +54,7 @@ def simple_app(environ, start_response):
     start_response('200 OK', [('Content-type', 'text/plain')])
     return ['The current value is: %s' % cache.get_value('value')]
 
+
 def using_none_app(environ, start_response):
     extra_args = {}
     clear = False
@@ -65,6 +91,23 @@ def cache_manager_app(environ, start_response):
         yield "test_key wasn't cleared, is: %s\n" % \
             cm.get_cache('test')['test_key']
 
+
+def test_session():
+    app = TestApp(SessionMiddleware(simple_session_app, data_dir='./cache', type='ext:memcached', url=mc_url))
+    res = app.get('/')
+    assert 'current value is: 1' in res
+    res = app.get('/')
+    assert 'current value is: 2' in res
+    res = app.get('/')
+    assert 'current value is: 3' in res
+
+
+def test_session_invalid():
+    app = TestApp(SessionMiddleware(simple_session_app, data_dir='./cache', type='ext:memcached', url=mc_url))
+    res = app.get('/invalid', headers=dict(Cookie='beaker.session.id=df7324911e246b70b5781c3c58328442; Path=/'))
+    assert 'current value is: 2' in res
+
+
 def test_has_key():
     cache = Cache('test', data_dir='./cache', url=mc_url, type='ext:memcached')
     o = object()
@@ -85,12 +128,12 @@ def test_dropping_keys():
     assert cache.has_key('fred')
     
     # Directly nuke the actual key, to simulate it being removed by memcached
-    cache._containers['test'].namespacemanager.mc.delete('test_test')
+    cache.namespace.mc.delete('test_test')
     assert not cache.has_key('test')
     assert cache.has_key('fred')
     
     # Nuke the keys dict, it might die, who knows
-    cache._containers['fred'].namespacemanager.mc.delete('test:keys')
+    cache.namespace.mc.delete('test:keys')
     assert cache.has_key('fred')
     
     # And we still need clear to work, even if it won't work well
@@ -101,7 +144,7 @@ def test_deleting_keys():
     cache.set_value('test', 20)
     
     # Nuke the keys dict, it might die, who knows
-    cache._containers['test'].namespacemanager.mc.delete('test:keys')
+    cache.namespace.mc.delete('test:keys')
     
     assert cache.has_key('test')
     
